#include <simura.h>

static Uint32 colortable[26] = {
	0x000080, 0x0000e0, 0x0000ff,
	0x008000, 0x00e000, 0x00ff00,
	0x008080, 0x00e0e0, 0x00ffff,
	0x800000, 0xe00000, 0xff0000,
	0x800080, 0xe000e0, 0xff00ff,
	0x808000, 0xe0e000, 0xffff00,
	0x808080, 0xe0e0e0, 0xffffff,
	0x76ca0a, 0x3cafaa, 0x60a848, 0x504858, 0x89ba43
};

Simura::Simura() {
  procbuf = NULL;

  supported[0] = false; /* 8bit depth */
  supported[1] = false;  /* 16 bit depth */
  supported[2] = false; /* 24 bit depth */
  supported[3] = true;  /* 32 bit depth */
  
  strcpy(name,"Simura");
  strcpy(author,"Fukuchi Kentarou, jaromil");
  version = 1;
}

void Simura::_delete() {
  func("Simura::_delete()");
  if(procbuf!=NULL) jfree(procbuf);
}

bool Simura::init() {
  func("Simura::init()");
  hheight = *h/2;
  hwidth = *w/2;
  color = mirror = 0;
  stat = 1;
  procbuf = jalloc(procbuf,*size);
  initialized = true;
  func("Simura initialized");
  return(initialized);
}

void *Simura::process(void *buffo) {
  switch(mirror) {
  case 1:
    mirror_l((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 2:
    mirror_r((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 3:
    mirror_d((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 4:
    mirror_dl((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 5:
    mirror_dr((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 6:
    mirror_u((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 7:
    mirror_ul((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 8:
    mirror_ur((Uint32*)buffo, (Uint32*)procbuf);
    break;
  case 0:
  default:
    mirror_no((Uint32*)buffo, (Uint32*)procbuf);
    break;
  }
  return(procbuf);
}

void Simura::mirror_no(Uint32 *src, Uint32 *dest) {
	unsigned int i;

	for(i=0; i<*size>>2; i++) {
		dest[i] = src[i] ^ color;
	}
}

bool Simura::kbd_input(SDL_keysym *keysym) {
  bool res = true;
  if(keysym->mod==KMOD_CAPS)
    switch(keysym->sym) {
    case SDLK_a:
    case SDLK_b:
    case SDLK_c:
    case SDLK_d:
    case SDLK_e:
    case SDLK_f:
    case SDLK_g:
    case SDLK_h:
    case SDLK_i:
    case SDLK_j:
    case SDLK_k:
    case SDLK_l:
    case SDLK_m:
    case SDLK_n:
    case SDLK_o:
    case SDLK_p:
    case SDLK_q:
    case SDLK_r:
    case SDLK_s:
    case SDLK_t:
    case SDLK_u:
    case SDLK_v:
    case SDLK_w:
    case SDLK_x:
    case SDLK_y:
    case SDLK_z:
      color = colortable[keysym->sym - SDLK_a];
      break;
    case SDLK_BACKSPACE:
      color = 0;
      break;
    default:
      res = false;
      break;
    }
  else
    switch(keysym->sym) {
    case SDLK_KP0:
      mirror = 0;
      break;
    case SDLK_KP1:
    case SDLK_KP2:
    case SDLK_KP3:
    case SDLK_KP4:
    case SDLK_KP5:
    case SDLK_KP6:
    case SDLK_KP7:
    case SDLK_KP8:
    case SDLK_KP9:
      mirror = keysym->sym - SDLK_KP1;
      break;
    default:
      res = false;
    }
  
  return(res);
}

void Simura::mirror_u(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=0; y<hheight; y++) {
		for(x=0; x<(*w); x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_d(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=hheight; y<(*h); y++) {
		for(x=0; x<(*w); x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_l(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=0; y<(*h); y++) {
		for(x=0; x<hwidth; x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_r(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=0; y<(*h); y++) {
		for(x=hwidth; x<(*w); x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_ul(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=0; y<hheight; y++) {
		for(x=0; x<hwidth; x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_ur(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=0; y<hheight; y++) {
		for(x=hwidth; x<(*w); x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_dl(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=hheight; y<(*h); y++) {
		for(x=0; x<hwidth; x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}

void Simura::mirror_dr(Uint32 *src, Uint32 *dest) {
	int x, y;

	for(y=hheight; y<(*h); y++) {
		for(x=hwidth; x<(*w); x++) {
			dest[y*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[y*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+x] = src[y*(*w)+x] ^ color;
			dest[((*h)-y-1)*(*w)+((*w)-x-1)] = src[y*(*w)+x] ^ color;
		}
	}
}
